#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <time.h>
#include <float.h>

/* Graficos com Allegro 5 */
#include "grafico.h"

#define FATOR_TIMER 1.0

#define CURSOR_SIZE 50
#define CURSOR_STEP 4

#define LATERAL_SOUND  "../audio/coin.wav"
#define TOPBOTTOM_SOUND  "../audio/coin-02.wav"

#define BMP1_BLOCK "../imagens/Bug_01_80x80.png"
#define BMP2_BLOCK "../imagens/Bug_02_80x80.png"
#define BMP3_BLOCK "../imagens/Enterprise_10.jpg"

//-----------------------------------------------------------------------------

int main(int argc, char *argv[])
{
  Window win;
  ALLEGRO_BITMAP *image, *image1, *image2;
  ALLEGRO_SAMPLE *aux_som;

  bool redraw = true, 
       sair = false;

  int bouncer_x, bouncer_y, new_width, new_height,
      bouncer_dx = CURSOR_STEP, 
      bouncer_dy = CURSOR_STEP,
      dxy_sinal;

  // Inicializa ambiente Allegro5
  win = graphinit(CURSOR_SIZE, CURSOR_SIZE);

  // Define velocidade do timer, que influi na percepção de movimento
  setTimerSpeed(win, FATOR_TIMER);
 
  // Carrega arquivos com soms.
  win.som = al_load_sample(LATERAL_SOUND); 
  aux_som = al_load_sample(TOPBOTTOM_SOUND);
  if (!win.som || !aux_som){
    fprintf(stderr, "Audio clip sample not loaded!\n" ); 
    exit(1);
  }

  // Cria 2 bitmaps com imagens
  image1 = imagemArq(BMP1_BLOCK, CURSOR_SIZE, CURSOR_SIZE, win);
  image2 = imagemArq(BMP3_BLOCK, CURSOR_SIZE, CURSOR_SIZE, win);
  image = image1;

  // Posição inicial do cursor que se movimentará
  bouncer_x = win.disp_data.width / 2 - CURSOR_SIZE / 2;
  bouncer_y = win.disp_data.height / 2 - CURSOR_SIZE / 2;

  /* Enquanto  a tecla ESC  não for  pressionada, irá executar
     o código especificado.
  */
  while (!sair) {

    // Trata redimensionamento de janela, guardando as novas dimensões
    al_acknowledge_resize(win.display);
    new_width = al_get_display_width(win.display);
    new_height = al_get_display_height(win.display);

    // Aguarda a ocorrência de eventos (movimento de mouse, 
    // pressionamento de teclas, timer)
    ALLEGRO_EVENT ev;
    al_wait_for_event(win.event_queue, &ev);
    
    // Se clicar com mouse no 'x' de fechar janela
    if(ev.type == ALLEGRO_EVENT_DISPLAY_CLOSE) {
      sair = true;
    }
    // Se movimentar o mouse
    else if(ev.type == ALLEGRO_EVENT_MOUSE_AXES ||
	    ev.type == ALLEGRO_EVENT_MOUSE_ENTER_DISPLAY) {
      
      bouncer_x = ev.mouse.x;
      bouncer_y = ev.mouse.y;
    }
    // Se soltar botões do mouse
    else if(ev.type == ALLEGRO_EVENT_MOUSE_BUTTON_UP) {
      image1 = image;
      image = image2;
      image2 = image1;
    }
    // A cada tick do timer
    else if(ev.type == ALLEGRO_EVENT_TIMER) {
 
      // Movimenta cursor, fazendo 'bate-e-volta' nas bordas da janela
      if(bouncer_x < 0 || bouncer_x > (new_width - CURSOR_SIZE)) {
	bouncer_dx = -bouncer_dx;
	if ( bouncer_x > (new_width - CURSOR_SIZE) )
	  bouncer_x = new_width - CURSOR_SIZE;
	
	al_play_sample(win.som, 1.0, 0.0, 1.0, ALLEGRO_PLAYMODE_ONCE, NULL);
      }
      
      if(bouncer_y < 0 || bouncer_y > (new_height - CURSOR_SIZE)) {
	bouncer_dy = -bouncer_dy;
	if ( bouncer_y > (new_height - CURSOR_SIZE) )
	  bouncer_y = new_height - CURSOR_SIZE;
	
	al_play_sample(aux_som, 1.0, 0.0, 1.0, ALLEGRO_PLAYMODE_ONCE, NULL);
      }

      bouncer_x += bouncer_dx;
      bouncer_y += bouncer_dy;

      redraw = true;
    }

    // Altera estado da tela apenas se houve mudança de imagens (redraw == True) ou
    // quando não houver eventos sendo tratados.
    if(redraw && al_is_event_queue_empty(win.event_queue)) {
      redraw = false;
      al_clear_to_color(VERDE);
      al_draw_bitmap(image, bouncer_x, bouncer_y, 0);
      al_flip_display();
    }
}

  // Antes de terminar, destroi bitmaps e sons criados
  al_destroy_bitmap(image);
  al_destroy_sample(aux_som);

  // Finaliza ambiente gráfico Allegro5
  graphdeinit(win);

  return 0;
}

